#!/usr/bin/env python3

import datetime


def list_of(type):
    """Return to_list such that to_list([x0, x1, ...]) returns
    [type(x0), type(x1), ...].
    """
    def to_list(x):
        return [type(e) for e in x]
    return to_list


def none_or(type):
    """Return to_type such that to_type(x) returns
    None if x is None else type(x).
    """
    def to_type(x):
        return None if x is None else type(x)
    return to_type


def to_datetime(x):
    """Convert a valid global date and time string
    (e.g. '2010-01-02T03:04:05Z') to the datetime.datetime object.
    """
    x = x.replace('Z', '+00:00').replace('T', ' ')
    x = x[:-3] + x[-2:]  # '...+XX:YY' -> '...+XXYY'
    try:
        return datetime.datetime.strptime(x, '%Y-%m-%d %H:%M:%S%z')
    except ValueError:
        return datetime.datetime.strptime(x, '%Y-%m-%d %H:%M:%S.%f%z')


def _repr(self, *attrs):
    name = self.__class__.__name__
    body = ', '.join('{attr}={value!r}'.format(attr=a, value=getattr(self, a))
                     for a in attrs)
    return '<{name} {body}>'.format(name=name, body=body)


class Status(object):
    """Entry object"""
    __slots__ = (
        'link', 'created_at', 'favorited', 'haiku_text', 'html', 'html_touch',
        'html_mobile', 'id', 'in_reply_to_status_id', 'in_reply_to_user_id',
        'keyword', 'replies', 'source', 'target', 'text', 'user',
    )

    def __init__(self, json):
        super().__init__()
        self.link = str(json['link'])
        self.created_at = to_datetime(json['created_at'])
        self.favorited = int(json['favorited'])
        self.haiku_text = none_or(str)(json.get('haiku_text'))
        self.html = none_or(str)(json.get('html'))
        self.html_touch = none_or(str)(json.get('html_touch'))
        self.html_mobile = none_or(str)(json.get('html_mobile'))
        self.id = str(json['id'])
        raw = json.get('in_reply_to_status_id')
        self.in_reply_to_status_id = none_or(str)(raw)
        raw = json.get('in_reply_to_user_id')
        self.in_reply_to_user_id = none_or(str)(raw)
        self.keyword = none_or(str)(json.get('keyword'))
        self.replies = none_or(list_of(Status))(json.get('replies'))
        self.source = str(json['source'])
        self.target = none_or(Target)(json.get('target'))
        self.text = none_or(str)(json.get('text'))
        self.user = User(json['user'])

    def __repr__(self):
        return _repr(self, 'link')


class User(object):
    """User object"""
    __slots__ = (
        'followers_count', 'name', 'id', 'profile_image_url', 'screen_name',
        'url',
    )

    def __init__(self, json):
        super().__init__()
        self.followers_count = int(json['followers_count'])
        self.name = str(json['name'])
        self.id = str(json['id'])
        self.profile_image_url = str(json['profile_image_url'])
        self.screen_name = str(json['screen_name'])
        self.url = str(json['url'])

    def __repr__(self):
        return _repr(self, 'id')


class Keyword(object):
    """Keyword object"""
    __slots__ = (
        'entry_count', 'followers_count', 'link', 'related_keywords', 'title',
        'word', 'url_name',
    )

    def __init__(self, json):
        super().__init__()
        self.entry_count = int(json['entry_count'])
        self.followers_count = int(json['followers_count'])
        self.link = str(json['link'])
        raw = json.get('related_keywords')
        self.related_keywords = none_or(list_of(str))(raw)
        self.title = str(json['title'])
        self.word = str(json['word'])
        self.url_name = none_or(str)(json.get('url_name'))

    def __repr__(self):
        return _repr(self, 'word')


class Target(object):
    """Target object"""
    __slots__ = ('title', 'word', 'url_name',)

    def __init__(self, json):
        super().__init__()
        self.title = str(json['title'])
        self.word = str(json['word'])
        self.url_name = none_or(str)(json.get('url_name'))

    def __repr__(self):
        return _repr(self, 'word')



#!/usr/bin/env python3

"""Hatena Haiku for Python 3

An unofficial library for the Hatena Haiku API

See also Hatena Haiku:

    http://h.hatena.ne.jp/

and the official API documents:

    http://developer.hatena.ne.jp/en/documents/haiku

Example:

import haiker
auth = haiker.OAuth(
    'MyConsumerKey', 'MyConsumerSecret',
    'MyAccessToken', 'MyAccessTokenSecret'
)
api = haiker.Haiker(auth)
for status in api.public_timeline(count=3):
    created_at = status.created_at.strftime('%Y-%m-%d %H:%M:%S')
    print(created_at, status.user.id, status.keyword)
"""

__version__ = '0.3.0'
__author__ = '@saltcandy123'
__license__ = 'MIT License'

from .api import Haiker
from .auth import BasicAuth, OAuth
from .error import HaikerError

__all__ = [
    'Haiker',
    'BasicAuth', 'OAuth',
    'HaikerError',
]



#!/usr/bin/env python3

import functools
import re
import requests
from . import error, types, utils


class BaseAPIHandler(object):
    """Base API handler

    Example:

    >>> h = haiker.api.BaseAPIHandler(
    ...         haiker.OAuth(
    ...             'MyConsumerKey', 'MyConsumerSecret',
    ...             'MyAccessToken', 'MyAccessTokenSecret'
    ...         ),
    ...         'http://h.hatena.ne.jp/api',
    ...         haiker.utils.DEFAULT_USER_AGENT
    ... )
    >>> statuses = h.get('/statuses/public_timeline.json', {'count': 3})
    >>> len(statuses)
    3
    >>> statuses[0]['created_at']
    '2010-01-02T03:04:05Z'
    >>> statuses[0]['user']['url']
    'http://h.hatena.ne.jp/xxxxxx/'
    """
    def __init__(self, auth, root, user_agent):
        super().__init__()
        self.auth = auth
        self.root = root
        self.user_agent = user_agent

    def _request(self, method, path, params=None, data=None, files=None):
        if re.search('[^a-zA-Z0-9./\\-_]|\\.\\.|//', path) is not None:
            raise ValueError('suspicious path: {0!r}'.format(path))
        url = self.root.rstrip('/') + '/' + path.lstrip('/')
        headers = {'User-Agent': self.user_agent}
        res = method(url, headers=headers, auth=self.auth,
                     params=utils.build_params(params),
                     data=utils.build_params(data),
                     files=files)
        res.raise_for_status()
        return res.json()

    def get(self, path, params=None):
        return self._request(requests.get, path, params=params)

    def post(self, path, params=None, data=None, files=None):
        return self._request(requests.post, path,
                             params=params, data=data, files=files)


class Haiker(object):
    """Handler for the Hatena Haiku RESTful API

    Example:

    >>> import haiker
    >>> auth = haiker.OAuth(
    ...     'MyConsumerKey', 'MyConsumerSecret',
    ...     'MyAccessToken', 'MyAccessTokenSecret'
    ... )
    >>> api = haiker.Haiker(auth)
    >>> statuses = api.public_timeline(count=3)
    >>> len(statuses)
    3
    >>> status = statuses[0]
    >>> status.created_at
    datetime.datetime(2010, 1, 2, 3, 4, 5, tzinfo=datetime.timezone.utc)
    >>> status.user.url
    'http://h.hatena.ne.jp/xxxxxx/'
    """
    @error.HaikerError.replace
    def __init__(self, auth=None, *,
                 user_agent=utils.DEFAULT_USER_AGENT,
                 root='http://h.hatena.ne.jp/api'):
        """auth is required to be None, a haiker.BasicAuth object
        or haiker.OAuth object.
        """
        super().__init__()
        self._handler = BaseAPIHandler(auth, root, user_agent)

    @property
    @error.HaikerError.replace
    def auth(self):
        return self._handler.auth

    @auth.setter
    @error.HaikerError.replace
    def auth(self, value):
        self._handler.auth = value

    # Timeline APIs
    @error.HaikerError.replace
    def public_timeline(self, *, body_formats=None, count=None, page=None,
                        since=None):
        """statuses/public_timeline"""
        params = utils.removed_dict(locals(), {'self'})
        path = '/statuses/public_timeline.json'
        res = self._handler.get(path, params)
        return types.list_of(types.Status)(res)

    @error.HaikerError.replace
    def keyword_timeline(self, word, *, count=None, page=None, since=None,
                         body_formats=None, sort=None):
        """statuses/keyword_timeline"""
        params = utils.removed_dict(locals(), {'self'})
        path = '/statuses/keyword_timeline.json'
        res = self._handler.get(path, params)
        return types.list_of(types.Status)(res)

    @error.HaikerError.replace
    def user_timeline(self, url_name=None, *, body_formats=None, count=None,
                      page=None, since=None, media=None, sort=None):
        """statuses/user_timeline"""
        params = utils.removed_dict(locals(), {'self', 'url_name'})
        if url_name is None:
            path = '/statuses/user_timeline.json'
        else:
            path = '/statuses/user_timeline/{0}.json'.format(url_name)
        res = self._handler.get(path, params)
        return types.list_of(types.Status)(res)

    @error.HaikerError.replace
    def friends_timeline(self, url_name=None, *, count=None, page=None,
                         since=None, body_formats=None):
        """statuses/friends_timeline"""
        params = utils.removed_dict(locals(), {'self', 'url_name'})
        if url_name is None:
            path = '/statuses/friends_timeline.json'
        else:
            path = '/statuses/friends_timeline/{0}.json'.format(url_name)
        res = self._handler.get(path, params)
        return types.list_of(types.Status)(res)

    @error.HaikerError.replace
    def album(self, *, body_formats=None, count=None, page=None,
              since=None, sort=None, word=None):
        """statuses/album"""
        params = utils.removed_dict(locals(), {'self'})
        path = '/statuses/album.json'
        res = self._handler.get(path, params)
        return types.list_of(types.Status)(res)

    # Entry and star APIs
    @error.HaikerError.replace
    def update_status(self, keyword, status, *, in_reply_to_status_id=None,
                      source=None, files=None, body_formats=None):
        """statuses/update"""
        params = utils.removed_dict(locals(), {'self', 'files'})
        if files is not None:
            files = [('file', f) for f in files]
        path = '/statuses/update.json'
        res = self._handler.post(path, None, params, files)
        return types.Status(res)

    @error.HaikerError.replace
    def show_status(self, eid, *, body_formats=None):
        """statuses/show"""
        params = utils.removed_dict(locals(), {'self', 'eid'})
        path = '/statuses/show/{0}.json'.format(eid)
        res = self._handler.get(path, params)
        return types.Status(res)

    @error.HaikerError.replace
    def delete_status(self, eid, author_url_name, *, body_formats=None):
        """statuses/destroy"""
        params = utils.removed_dict(locals(), {'self', 'eid'})
        path = '/statuses/destroy/{0}.json'.format(eid)
        res = self._handler.post(path, params)
        return types.Status(res)

    @error.HaikerError.replace
    def add_star(self, eid, *, body_formats=None):
        """favorites/create"""
        params = utils.removed_dict(locals(), {'self', 'eid'})
        path = '/favorites/create/{0}.json'.format(eid)
        res = self._handler.post(path, params)
        return types.Status(res)

    @error.HaikerError.replace
    def remove_star(self, eid, *, body_formats=None):
        """favorites/destroy"""
        params = utils.removed_dict(locals(), {'self', 'eid'})
        path = '/favorites/destroy/{0}.json'.format(eid)
        res = self._handler.post(path, params)
        return types.Status(res)

    # User and keyword APIs
    @error.HaikerError.replace
    def show_user(self, url_name=None):
        """friendships/show"""
        if url_name is None:
            path = '/friendships/show.json'.format(url_name)
        else:
            path = '/friendships/show/{0}.json'.format(url_name)
        res = self._handler.get(path)
        return types.User(res)

    @error.HaikerError.replace
    def show_keyword(self, word, *, without_related_keywords=None):
        """keywords/show"""
        params = utils.removed_dict(locals(), {'self'})
        path = '/keywords/show.json'
        res = self._handler.get(path, params)
        return types.Keyword(res)

    @error.HaikerError.replace
    def hot_keywords(self, *, without_related_keywords=None):
        """keywords/hot"""
        params = utils.removed_dict(locals(), {'self'})
        path = '/keywords/hot.json'
        res = self._handler.get(path, params)
        return types.list_of(types.Keyword)(res)

    @error.HaikerError.replace
    def keyword_list(self, *, page=None, without_related_keywords=None,
                     word=None):
        """keywords/list"""
        params = utils.removed_dict(locals(), {'self'})
        path = '/keywords/list.json'
        res = self._handler.get(path, params)
        return types.list_of(types.Keyword)(res)

    @error.HaikerError.replace
    def associate_keywords(self, word1, word2, *,
                           without_related_keywords=None):
        """keywords/relation/create"""
        params = utils.removed_dict(locals(), {'self'})
        path = '/keywords/relation/create.json'
        res = self._handler.post(path, params)
        return types.Keyword(res)

    @error.HaikerError.replace
    def dissociate_keywords(self, word1, word2, *,
                            without_related_keywords=None):
        """keywords/relation/destroy"""
        params = utils.removed_dict(locals(), {'self'})
        path = '/keywords/relation/destroy.json'
        res = self._handler.post(path, params)
        return types.Keyword(res)

    # Favorite APIs
    @error.HaikerError.replace
    def friends(self, url_name=None, *, page=None):
        """statuses/friends"""
        params = utils.removed_dict(locals(), {'self', 'url_name'})
        if url_name is None:
            path = '/statuses/friends.json'
        else:
            path = '/statuses/friends/{0}.json'.format(url_name)
        res = self._handler.get(path, params)
        return types.list_of(types.User)

    @error.HaikerError.replace
    def followers(self, url_name=None, *, page=None):
        """statuses/followers"""
        params = utils.removed_dict(locals(), {'self', 'url_name'})
        if url_name is None:
            path = '/statuses/followers.json'
        else:
            path = '/statuses/followers/{0}.json'.format(url_name)
        res = self._handler.get(path, params)
        return types.list_of(types.User)

    @error.HaikerError.replace
    def add_friend(self, url_name):
        """friendships/create"""
        path = '/friendships/create/{0}.json'.format(url_name)
        res = self._handler.post(path)
        return types.User(res)

    @error.HaikerError.replace
    def remove_friend(self, url_name):
        """friendships/destroy"""
        path = '/friendships/destroy/{0}.json'.format(url_name)
        res = self._handler.post(path)
        return types.User(res)

    @error.HaikerError.replace
    def favorite_keywords(self, url_name=None, *, page=None,
                          without_related_keywords=None):
        """statuses/keywords"""
        params = utils.removed_dict(locals(), {'self', 'url_name'})
        if url_name is None:
            path = '/statuses/keywords.json'
        else:
            path = '/statuses/keywords/{0}.json'.format(url_name)
        res = self._handler.get(path, params)
        return types.list_of(types.Keyword)

    @error.HaikerError.replace
    def add_favorite_keyword(self, word, *, without_related_keywords=None):
        """keywords/create"""
        params = utils.removed_dict(locals(), {'self'})
        path = '/keywords/create.json'
        res = self._handler.post(path, params)
        return types.Keyword(res)

    @error.HaikerError.replace
    def remove_favorite_keyword(self, word, *, without_related_keywords=None):
        """keywords/destroy"""
        params = utils.removed_dict(locals(), {'self'})
        path = '/keywords/destroy.json'
        res = self._handler.post(path, params)
        return types.Keyword(res)



#!/usr/bin/env python3

import functools


class HaikerError(Exception):
    """Raised when something has gone wrong."""
    def __init__(self, error):
        super().__init__()
        self._error = error

    @property
    def causal_error(self):
        """An exception object which has raised this error"""
        return self._error

    def __str__(self):
        return str(self._error)

    @classmethod
    def replace(cls, func):
        """Return a function which has the same behavior as func but
        raises this exception instead of raising other exception.
        """
        @functools.wraps(func)
        def call(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except cls:
                raise
            except Exception as e:
                raise cls(e) from e
        return call



#!/usr/bin/env python3

import collections
import datetime
import time
from . import __version__


DEFAULT_USER_AGENT = 'python-haiker/{0}'.format(__version__)


def removed_dict(dic, keys):
    """Create a copy of dic, remove keys and return it."""
    return {kw: dic[kw] for kw in dic if kw not in keys}


def strftime(date, date_format):
    """Return a string expession of date.  This function is
    like date.strftime(date_format) but adjusts the timezone
    to UTC.
    """
    try:
        date = date.astimezone(datetime.timezone.utc)
    except ValueError:
        date = time.gmtime(time.mktime(date.timetuple()))
        return time.strftime(date_format, date)
    return date.strftime(date_format)


def serialize(obj, *, charset='utf-8',
              date_format='%a, %d %B %Y %H:%M:%S GMT'):
    """Return a string expression of obj."""
    if obj is None:
        return None
    if isinstance(obj, int):  # bool is a subclass of int
        obj = str(int(obj))
    if isinstance(obj, datetime.datetime):
        obj = strftime(obj, date_format)
    if isinstance(obj, str):
        obj = obj.encode(charset)
    if isinstance(obj, bytes):
        return obj
    if isinstance(obj, collections.Iterable):
        return b','.join(b'' if d is None else serialize(d) for d in obj)
    raise TypeError('{0!r} is an unsupported type'.format(type(obj)))


def build_params(params):
    """Return a list of (key, serialized value) pairs from params."""
    if params is None:
        return None
    if isinstance(params, collections.Mapping):
        return [(key, serialize(params[key])) for key in params]
    return [(key, serialize(value)) for key, value in params]



#!/usr/bin/env python3

import os.path
import re
import sys
import setuptools


if sys.version_info < (3, 2):
    raise RuntimeError('Python 3.2 or greater is required')


def relative_path(path):
    return os.path.join(os.path.dirname(__file__), path)


def long_description():
    with open(relative_path('README.rst')) as f:
        return f.read()


def from_init(name, default=None):
    with open(relative_path('haiker/__init__.py')) as f:
        for line in f:
            m = re.search('(^|\\s){0}\\s*=\\s*(.+)'.format(name), line)
            if m is not None:
                return eval(m.group(2))
    return default


setuptools.setup(
    name='hatena-haiker',
    version=from_init('__version__'),
    author=from_init('__author__'),
    author_email='saltcandy123+haiker@gmail.com',
    url='https://github.com/saltcandy123/hatena-haiker/',
    description='Hatena Haiku for Python 3',
    long_description=long_description(),
    license=from_init('__license__'),
    packages=['haiker'],
    install_requires=[
        'requests>=0',
        'requests-oauthlib>=0',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Software Development :: Libraries',
    ],
    keywords='hatena haiku api',
)



#!/usr/bin/env python3

import urllib.parse
import requests
import requests_oauthlib
from . import error, utils


class BasicAuth(requests.auth.HTTPBasicAuth):
    """Basic Auth handler

    Example:

    >>> auth = haiker.BasicAuth('MyUsername', 'MyPassword')
    >>> api = haiker.Haiker(auth)
    """


class OAuth(object):
    """OAuth handler

    Example 1:

    >>> auth = haiker.OAuth(
    ...     'MyConsumerKey', 'MyConsumerSecret',
    ...     'MyAccessToken', 'MyAccessTokenSecret'
    ... )
    >>> api = haiker.Haiker(auth)

    Example 2:

    >>> auth = haiker.OAuth('MyConsumerKey', 'MyConsumerSecret')
    >>> auth.initiate(['read_public', 'write_public'])
    ('MyRequestToken', 'MyRequestSecret')
    >>> print('GOTO: {url}'.format(url=auth.auth_url()))
    GOTO: https://www.hatena.ne.jp/xxxxx/xxxx?xxx=xxxx
    >>> verifier = 'MyVerifierCode'
    >>> auth.verify(verifier)
    ('MyAccessToken', 'MyAccessTokenSecret')
    >>> api = haiker.Haiker(auth)
    """
    @error.HaikerError.replace
    def __init__(self, consumer_key, consumer_secret=None,
                 oauth_token=None, oauth_token_secret=None, *,
                 user_agent=utils.DEFAULT_USER_AGENT):
        super().__init__()
        self._keys = {
            'client_key': consumer_key,
            'client_secret': consumer_secret,
            'resource_owner_key': oauth_token,
            'resource_owner_secret': oauth_token_secret,
        }
        self.user_agent = user_agent
        self._auth = self._make_auth(**self._keys)

    def __call__(self, *args, **kwargs):
        return self._auth.__call__(*args, **kwargs)

    @error.HaikerError.replace
    def initiate(self, scope, callback_url='oob', *,
                 url='https://www.hatena.com/oauth/initiate'):
        """Fetch a request token pair."""
        self._keys['resource_owner_key'] = None
        self._keys['resource_owner_secret'] = None
        auth = self._make_auth(callback_uri=callback_url, **self._keys)
        return self._receive_token(url, auth, data={'scope': scope})

    @error.HaikerError.replace
    def auth_url(self, *, url='https://www.hatena.ne.jp/oauth/authorize'):
        """Get a URL to authorize.  You can find the verifier code
        to visit the URL.
        """
        key = self._keys['resource_owner_key']
        if key is None:
            clsname = self.__class__.__name__
            msg = 'calling {0}.initiate() before is required'.format(clsname)
            raise RuntimeError(msg)
        quoted = urllib.parse.quote(key)
        return '{url}?oauth_token={query}'.format(url=url, query=quoted)

    @error.HaikerError.replace
    def verify(self, oauth_verifier, *,
               url='https://www.hatena.com/oauth/token'):
        """Fetch an access token pair.  This pair can be two arguments
        in OAuth(): oauth_token and oauth_token_secret.
        """
        auth = self._make_auth(verifier=oauth_verifier, **self._keys)
        return self._receive_token(url, auth)

    def _make_auth(self, **kwargs):
        return requests_oauthlib.OAuth1(**kwargs)

    def _receive_token(self, url, auth, data=None):
        headers = {'User-Agent': self.user_agent}
        res = requests.post(url, auth=auth, headers=headers,
                            data=utils.build_params(data))
        res.raise_for_status()
        d = urllib.parse.parse_qs(res.text)
        token = d['oauth_token'][0]
        token_secret = d['oauth_token_secret'][0]
        self._keys['resource_owner_key'] = token
        self._keys['resource_owner_secret'] = token_secret
        self._auth = self._make_auth(**self._keys)
        return token, token_secret
